@@ -75,12 +75,14 @@
 
 struct GsdWacomManagerPrivate
 {
+	guint start_idle_id;
         GSettings *wacom_settings;
         GSettings *stylus_settings;
         GSettings *eraser_settings;
         GSettings *cursor_settings;
         GSettings *pad_settings;
         GdkDeviceManager *device_manager;
+        guint device_added_id;
 };
 
 static void     gsd_wacom_manager_class_init  (GsdWacomManagerClass *klass);
@@ -100,9 +102,6 @@ gsd_wacom_manager_constructor (GType                     type,
                               GObjectConstructParam     *construct_properties)
 {
         GsdWacomManager      *wacom_manager;
-        GsdWacomManagerClass *klass;
-
-        klass = GSD_WACOM_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_WACOM_MANAGER));
 
         wacom_manager = GSD_WACOM_MANAGER (G_OBJECT_CLASS (gsd_wacom_manager_parent_class)->constructor (type,
                                                                                                          n_construct_properties,
@@ -114,10 +113,6 @@ gsd_wacom_manager_constructor (GType                     type,
 static void
 gsd_wacom_manager_dispose (GObject *object)
 {
-        GsdWacomManager *wacom_manager;
-
-        wacom_manager = GSD_WACOM_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_wacom_manager_parent_class)->dispose (object);
 }
 
@@ -150,8 +145,8 @@ set_devicepresence_handler (GsdWacomManager *manager)
         if (device_manager == NULL)
                 return;
 
-        g_signal_connect (G_OBJECT (device_manager), "device-added",
-                          G_CALLBACK (device_added_cb), manager);
+        manager->priv->device_added_id = g_signal_connect (G_OBJECT (device_manager), "device-added",
+                                                           G_CALLBACK (device_added_cb), manager);
         manager->priv->device_manager = device_manager;
 }
 
@@ -580,6 +575,8 @@ gsd_wacom_manager_idle_cb (GsdWacomManager *manager)
 
         gnome_settings_profile_end (NULL);
 
+        manager->priv->start_idle_id = 0;
+
         return FALSE;
 }
 
@@ -589,7 +586,7 @@ gsd_wacom_manager_start (GsdWacomManager *manager,
 {
         gnome_settings_profile_start (NULL);
 
-        g_idle_add ((GSourceFunc) gsd_wacom_manager_idle_cb, manager);
+        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) gsd_wacom_manager_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -604,7 +601,7 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
         g_debug ("Stopping wacom manager");
 
         if (p->device_manager != NULL) {
-                g_object_unref (p->device_manager);
+                g_signal_handler_disconnect (p->device_manager, p->device_added_id);
                 p->device_manager = NULL;
         }
 
@@ -646,6 +643,9 @@ gsd_wacom_manager_finalize (GObject *object)
 
         g_return_if_fail (wacom_manager->priv != NULL);
 
+        if (wacom_manager->priv->start_idle_id != 0)
+                g_source_remove (wacom_manager->priv->start_idle_id);
+
         G_OBJECT_CLASS (gsd_wacom_manager_parent_class)->finalize (object);
 }
 
